"use strict";
/**
 * Copyright 2021 Google LLC.
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.CdpConnection = void 0;
const cdpClient_1 = require("./cdpClient");
const log_1 = require("../utils/log");
const logCdp = (0, log_1.log)('cdp');
/**
 * Represents a high-level CDP connection to the browser backend.
 * Manages a CdpClient instance for each active CDP session.
 */
class CdpConnection {
    _transport;
    _browserCdpClient;
    _sessionCdpClients = new Map();
    _commandCallbacks = new Map();
    _nextId;
    constructor(_transport) {
        this._transport = _transport;
        this._nextId = 0;
        this._transport.setOnMessage(this._onMessage);
        this._browserCdpClient = (0, cdpClient_1.createClient)(this, null);
    }
    /**
     * Close the connection to the browser.
     */
    close() {
        this._transport.close();
        for (const [_id, { reject }] of this._commandCallbacks) {
            reject(new Error('Disconnected'));
        }
        this._commandCallbacks.clear();
        this._sessionCdpClients.clear();
    }
    /**
     * @returns The CdpClient object attached to the root browser session.
     */
    browserClient() {
        return this._browserCdpClient;
    }
    /**
     * Get a CdpClient instance by sessionId.
     * @param sessionId The sessionId of the CdpClient to retrieve.
     * @returns The CdpClient object attached to the given session, or null if the session is not attached.
     */
    getCdpClient(sessionId) {
        const cdpClient = this._sessionCdpClients.get(sessionId);
        if (!cdpClient) {
            throw new Error('Unknown CDP session ID');
        }
        return cdpClient;
    }
    sendCommand(method, params, sessionId) {
        return new Promise((resolve, reject) => {
            const id = this._nextId++;
            this._commandCallbacks.set(id, { resolve, reject });
            let messageObj = { id, method, params };
            if (sessionId) {
                messageObj.sessionId = sessionId;
            }
            const messageStr = JSON.stringify(messageObj);
            this._transport.sendMessage(messageStr);
            logCdp('sent > ' + messageStr);
        });
    }
    _onMessage = async (message) => {
        logCdp('received < ' + message);
        const parsed = JSON.parse(message);
        // Update client map if a session is attached or detached.
        // Listen for these events on every session.
        if (parsed.method === 'Target.attachedToTarget') {
            const { sessionId } = parsed.params;
            this._sessionCdpClients.set(sessionId, (0, cdpClient_1.createClient)(this, sessionId));
        }
        else if (parsed.method === 'Target.detachedFromTarget') {
            const { sessionId } = parsed.params;
            const client = this._sessionCdpClients.get(sessionId);
            if (client) {
                this._sessionCdpClients.delete(sessionId);
            }
        }
        if (parsed.id !== undefined) {
            // Handle command response.
            const callbacks = this._commandCallbacks.get(parsed.id);
            if (callbacks) {
                if (parsed.result) {
                    callbacks.resolve(parsed.result);
                }
                else if (parsed.error) {
                    callbacks.reject(parsed.error);
                }
            }
        }
        else if (parsed.method) {
            const client = parsed.sessionId
                ? this._sessionCdpClients.get(parsed.sessionId)
                : this._browserCdpClient;
            if (client) {
                client._onCdpEvent(parsed.method, parsed.params || {});
            }
        }
    };
}
exports.CdpConnection = CdpConnection;
//# sourceMappingURL=cdpConnection.js.map